@@ -374,7 +374,11 @@ OutgoingMessage.prototype._storeHeader = function (firstLine, headers) {
 
       if (connectionExpression.test(field)) {
         sentConnectionHeader = true;
-        if (closeExpression.test(value)) this._last = true;
+        if (closeExpression.test(value)) {
+          this._last = true;
+        } else {
+          this.shouldKeepAlive = true;
+        }
 
       } else if (transferEncodingExpression.test(field)) {
         sentTransferEncodingHeader = true;
@@ -429,7 +433,8 @@ OutgoingMessage.prototype.write = function (chunk, encoding) {
   }
 
   if (!this._hasBody) {
-    throw new Error("This type of response MUST NOT have a body.");
+    console.error("This type of response MUST NOT have a body. Ignoring write() calls.");
+    return true;
   }
 
   if (typeof chunk !== "string"
@@ -601,7 +606,7 @@ ServerResponse.prototype.writeHeader = function () {
 function ClientRequest (socket, method, url, headers) {
   OutgoingMessage.call(this, socket);
 
-  this.method = method;
+  this.method = method = method.toUpperCase();
   this.shouldKeepAlive = false;
   if (method === "GET" || method === "HEAD") {
     this.useChunkedEncodingByDefault = false;
@@ -855,7 +860,13 @@ function Client ( ) {
         // For the moment we reconnect for every request. FIXME!
         // All that should be required for keep-alive is to not reconnect,
         // but outgoingFlush instead.
-        self.end();
+        if (req.shouldKeepAlive) {
+          outgoingFlush(self)
+          self._outgoing.shift()
+          outgoingFlush(self)
+        } else {
+          self.end();
+        }
       });
 
       req.emit("response", res);
@@ -906,7 +917,7 @@ function Client ( ) {
   });
 
   self.onend = function () {
-    parser.finish();
+    if (parser) parser.finish();
     debug("self got end closing. readyState = " + self.readyState);
     self.end();
   };
