@@ -1131,8 +1131,8 @@ Server.prototype.listen = function () {
           throw r;
         }
       } else {
-        if (!r.isFile()) {
-          throw new Error("Non-file exists at  " + path);
+        if (!r.isSocket()) {
+          throw new Error("Non-socket exists at  " + path);
         } else {
           fs.unlink(path, function (err) {
             if (err) {
