@@ -16,12 +16,13 @@ var stdio = process.binding('stdio');
 
 
 
-exports.createInterface = function (output) {
-  return new Interface(output);
+exports.createInterface = function (output, completer) {
+  return new Interface(output, completer);
 };
 
-function Interface (output) {
+function Interface (output, completer) {
   this.output = output;
+  this.completer = completer;
 
   this.setPrompt("node> ");
 
@@ -40,7 +41,6 @@ function Interface (output) {
     // Check process.env.TERM ?
     stdio.setRawMode(true);
     this._tty = true;
-    this.columns = stdio.getColumns();
 
     // Cursor position on the line.
     this.cursor = 0;
@@ -52,6 +52,11 @@ function Interface (output) {
 
 inherits(Interface, EventEmitter);
 
+Interface.prototype.__defineGetter__("columns", function () {
+  if (this._tty) {
+    return stdio.getColumns();
+  }
+});
 
 Interface.prototype.setPrompt = function (prompt, length) {
   this._prompt = prompt;
@@ -126,6 +131,82 @@ Interface.prototype._normalWrite = function (b) {
   this.emit('line', b.toString());
 };
 
+Interface.prototype._insertString = function (c) {
+  //BUG: Problem when adding tabs with following content.
+  //     Perhaps the bug is in _refreshLine(). Not sure.
+  //     A hack would be to insert spaces instead of literal '\t'.
+  if (this.cursor < this.line.length) {
+    var beg = this.line.slice(0, this.cursor);
+    var end = this.line.slice(this.cursor, this.line.length);
+    this.line = beg + c + end;
+    this.cursor += c.length;
+    this._refreshLine();
+  } else {
+    this.line += c;
+    this.cursor += c.length;
+    this.output.write(c);
+  }
+};
+
+Interface.prototype._tabComplete = function () {
+  var self = this;
+
+  var rv = this.completer(self.line.slice(0, self.cursor));
+  var completions = rv[0],
+      completeOn = rv[1];  // the text that was completed
+  if (completions && completions.length) {
+    // Apply/show completions.
+    if (completions.length === 1) {
+      self._insertString(completions[0].slice(completeOn.length));
+      self._refreshLine();
+    } else {
+      //TODO: If there is a common prefix to all matches (e.g. Python `sys.exi<Tab>`) then apply that portion
+      self.output.write("\r\n");
+      var width = completions.reduce(function(a, b) {
+        return a.length > b.length ? a : b;
+      }).length + 2;  // 2 space padding
+      var maxColumns = Math.floor(this.columns / width) || 1;
+
+      function handleGroup(group) {
+        if (group.length == 0) {
+          return;
+        }
+        var minRows = Math.ceil(group.length / maxColumns);
+        for (var row = 0; row < minRows; row++) {
+          for (var col = 0; col < maxColumns; col++) {
+            var idx = row * maxColumns + col;
+            if (idx >= group.length) {
+              break;
+            }
+            var item = group[idx];
+            self.output.write(item);
+            if (col < maxColumns - 1) {
+              for (var s = 0; s < width - item.length; s++) {
+                self.output.write(' ');
+              }
+            }
+          }
+          self.output.write('\r\n');
+        }
+        self.output.write('\r\n');
+      }
+
+      var group = [], c;
+      for (var i = 0; i < completions.length; i++) {
+        c = completions[i];
+        if (c === "") {
+          handleGroup(group);
+          group = [];
+        } else {
+          group.push(c);
+        }
+      }
+      handleGroup(group);
+      self._refreshLine();
+    }
+  }
+};
+
 Interface.prototype._historyNext = function () {
   if (this.historyIndex > 0) {
     this.historyIndex--;
@@ -228,6 +309,12 @@ Interface.prototype._ttyWrite = function (b) {
       this._historyNext();
       break;
 
+    case 9: // tab, completion
+      if (this.completer) {
+        this._tabComplete();
+      }
+      break;
+
     case 16: // control-p, previous history item
       this._historyPrev();
       break;
@@ -251,6 +338,12 @@ Interface.prototype._ttyWrite = function (b) {
           this.cursor++;
           this.output.write('\x1b[0C');
         }
+      } else if (b[1] === 91 && b[2] === 72) { // home
+        this.cursor = 0;
+        this._refreshLine();
+      } else if (b[1] === 91 && b[2] === 70) { // end
+        this.cursor = this.line.length;
+        this._refreshLine();
       } else if (b[1] === 91 && b[2] === 65) { // up arrow
         this._historyPrev();
       } else if (b[1] === 91 && b[2] === 66) { // down arrow
@@ -264,18 +357,15 @@ Interface.prototype._ttyWrite = function (b) {
 
     default:
       var c = b.toString('utf8');
-      if (this.cursor < this.line.length) {
-        var beg = this.line.slice(0, this.cursor);
-        var end = this.line.slice(this.cursor, this.line.length);
-        this.line = beg + c + end;
-        this.cursor += c.length;
-        this._refreshLine();
-      } else {
-        this.line += c;
-        this.cursor += c.length;
-        this.output.write(c);
+      var lines = c.split(/\r\n|\n|\r/);
+      for (var i = 0; i < lines.length; i++) {
+        if (i > 0) {
+          this._ttyWrite(new Buffer([13]));
+        }
+        this._insertString(lines[i]);
       }
       break;
   }
 };
 
+exports.Interface = Interface;
