@@ -1,4 +1,5 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 var fs = require('fs');
 var got_error = false;
 var success_count = 0;
@@ -23,7 +24,40 @@ fs.lstat(".", function (err, stats) {
   }
 });
 
-puts("stating: " + __filename);
+// fstat
+fs.open(".", "r", undefined, function(err, fd) {
+  assert.ok(!err);
+  assert.ok(fd);
+
+  fs.fstat(fd, function (err, stats) {
+    if (err) {
+      got_error = true;
+    } else {
+      p(stats);
+      assert.ok(stats.mtime instanceof Date);
+      success_count++;
+      fs.close(fd);
+    }
+  });
+});
+
+// fstatSync
+fs.open(".", "r", undefined, function(err, fd) {
+  var stats;
+  try {
+    stats = fs.fstatSync(fd);
+  } catch (err) {
+    got_error = true;
+  }
+  if (stats) {
+    p(stats);
+    assert.ok(stats.mtime instanceof Date);
+    success_count++;
+  }
+  fs.close(fd);
+});
+
+console.log("stating: " + __filename);
 fs.stat(__filename, function (err, s) {
   if (err) {
     got_error = true;
@@ -31,25 +65,25 @@ fs.stat(__filename, function (err, s) {
     p(s);
     success_count++;
 
-    puts("isDirectory: " + JSON.stringify( s.isDirectory() ) );
+    console.log("isDirectory: " + JSON.stringify( s.isDirectory() ) );
     assert.equal(false, s.isDirectory());
 
-    puts("isFile: " + JSON.stringify( s.isFile() ) );
+    console.log("isFile: " + JSON.stringify( s.isFile() ) );
     assert.equal(true, s.isFile());
 
-    puts("isSocket: " + JSON.stringify( s.isSocket() ) );
+    console.log("isSocket: " + JSON.stringify( s.isSocket() ) );
     assert.equal(false, s.isSocket());
 
-    puts("isBlockDevice: " + JSON.stringify( s.isBlockDevice() ) );
+    console.log("isBlockDevice: " + JSON.stringify( s.isBlockDevice() ) );
     assert.equal(false, s.isBlockDevice());
 
-    puts("isCharacterDevice: " + JSON.stringify( s.isCharacterDevice() ) );
+    console.log("isCharacterDevice: " + JSON.stringify( s.isCharacterDevice() ) );
     assert.equal(false, s.isCharacterDevice());
 
-    puts("isFIFO: " + JSON.stringify( s.isFIFO() ) );
+    console.log("isFIFO: " + JSON.stringify( s.isFIFO() ) );
     assert.equal(false, s.isFIFO());
 
-    puts("isSymbolicLink: " + JSON.stringify( s.isSymbolicLink() ) );
+    console.log("isSymbolicLink: " + JSON.stringify( s.isSymbolicLink() ) );
     assert.equal(false, s.isSymbolicLink());
 
     assert.ok(s.mtime instanceof Date);
@@ -57,7 +91,7 @@ fs.stat(__filename, function (err, s) {
 });
 
 process.addListener("exit", function () {
-  assert.equal(3, success_count);
+  assert.equal(5, success_count);
   assert.equal(false, got_error);
 });
 
