@@ -1,4 +1,5 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 var path = require('path');
 var fs = require('fs');
 
@@ -11,16 +12,16 @@ var rmdir_error = false;
 
 fs.mkdir(d, 0666, function (err) {
   if (err) {
-    puts("mkdir error: " + err.message);
+    console.log("mkdir error: " + err.message);
     mkdir_error = true;
   } else {
-    puts("mkdir okay!");
+    console.log("mkdir okay!");
     fs.rmdir(d, function (err) {
       if (err) {
-        puts("rmdir error: " + err.message);
+        console.log("rmdir error: " + err.message);
         rmdir_error = true;
       } else {
-        puts("rmdir okay!");
+        console.log("rmdir okay!");
       }
     });
   }
@@ -29,5 +30,5 @@ fs.mkdir(d, 0666, function (err) {
 process.addListener("exit", function () {
   assert.equal(false, mkdir_error);
   assert.equal(false, rmdir_error);
-  puts("exit");
+  console.log("exit");
 });
